%Rotina que exibe o mapeamento entrada-sada e a fronteira de deciso de
%uma rede MLP aplicada ao problema de equalizao

%Entradas:
%   canal - vetor com os coeficientes do canal (1 x nc)
%   atraso - atraso de equalizao, i.e., s(n-atraso) dever ser recuperado na sada da rede
%   W1 - matriz de pesos da camada de entrada da MLP - (N x K+1)
%       N - nmero de neurnios na camada intermediria
%       K  o nmero de entradas externas (no caso, para haver visualizao do mapeamento, K = 2)
%   W2 - matriz de pesos da camada de sada da MLP - (1 x N+1)
%       No caso geral, seria L x N+1, onde L  o nmero de sadas da rede
%   entrada - matriz com os dados de entrada do conjunto de treinamento (K x NT), onde NT denota o nmero de padres (amostras) de entrada

function map_fronteira_mlp_equalizacao(canal,atraso,W1,W2,entrada)

%nmero de coeficientes do canal
nc = length(canal);
%nmero de entradas da rede = dimenso dos estados do canal (subtrai o bias)
K = size(W1,2); K = K - 1; 
%obtm os estados do canal
[C,S] = channel_states(canal,K);
%Intervalo de anlise do mapeamento
x0 = (min(min(C))-1:.01:max(max(C))+1); MM = max(size(x0)); x1 = x0;
%MAPEAMENTO ENTRADA-SADA DA REDE
mlp = zeros(MM,MM);
for jj = 1:MM
    for ii = 1:MM
        xx = [x0(jj); x1(ii); 1]; %[r(k) r(k-1) bias]
        %sada da rede para a entrada xx
        mlp(ii,jj) = W2*[tanh(W1*xx);1];
    end
end
figure; mesh(x0,x1,mlp); xlabel('r(n)'); ylabel('r(n-1)'); zlabel('y_{MLP}(n)'); colormap jet;
hold on;
for ii=1:2^(K+nc-1)
     if S(ii,atraso+1) == 1
        plot3(C(1,ii),C(2,ii),1,'k+','MarkerSize',8,'LineWidth',2);
     else
        plot3(C(1,ii),C(2,ii),-1,'k*','MarkerSize',8,'LineWidth',2); 
     end
end
%VISUALIZAO DO CENRIO TRATADO E FRONTEIRA DE DECISO
figure; 
plot(entrada(1,:),entrada(2,:),'r.');
hold on;  
for ii=1:2^(K+nc-1)
     if S(ii,atraso+1) == 1
        p1 = plot(C(1,ii),C(2,ii),'k+','MarkerSize',8,'LineWidth',2);
     else
        p2 = plot(C(1,ii),C(2,ii),'ko','MarkerSize',8,'LineWidth',2); 
     end
end
legend([p1 p2],'s(n-d) = +1','s(n-d) = -1');
xlabel('r(n)'); ylabel('r(n-1)'); contour(x0,x1,mlp,[0 0],'k');


